import {configPerson} from '../model/configPerson';

export class Person {
    //attributes
    private readonly dni: string;
    private name: string;
    private address: string;
    private phone: number;
    private birthDate: Date;  //Date is an type of Object in Javascript

    /* In ES, constructor can't be overloaded, so in Typescript, we can use
    *  type validation to determinate the actions to do
    */
    constructor(input: configPerson | null) {
        if (input) {
            this.name = input.name;
            this.dni = input.dni;
            this.address = input.address;
            this.birthDate = input.birthDate;
            this.phone = input.phone;
        } else {
            this.dni = "";
            this.name = "";
            this.address = "";
            this.phone = 0;
            this.birthDate = new Date("");
        }

    }

    //methods
    public getDni(): string {
        return this.dni;
    }
    public setName(newName: string): void {
        this.name = newName;
    }
    public getName(): string {
        return this.name;
    }
    public setPhone(newPhone: number): void {
        this.phone = newPhone;
    }
    public getPhone(): number {
        return this.phone;
    }
    public setBirthDate(newBD: Date): void {
        this.birthDate = newBD;
    }
    public getAge(): number {
        const ageDifMs = Date.now() - this.birthDate.getTime();
        const ageDate = new Date(ageDifMs); // miliseconds from epoch
        return Math.abs(ageDate.getUTCFullYear() - 1970);
    }
    public toString(): string {
        return "DNI: " + this.dni + ", NAME: " + this.name + ", AGE: " + this.getAge();
    }

}