/**
 * Decorators on properties
 * @param target
 * @param propertyKey
 */
export function validate(target: any, propertyKey: string) {
  let value = target[propertyKey]; //read de value of this variable
  Object.defineProperty(target, propertyKey, {
    //changing descriptor
    get: () => value,
    set: newValue => {
      const pattern = /^[0-9]{4}\s?[A-Z]{3}$/;
      if (pattern.test(newValue)) {
        value = newValue;
      } else {
        value = "Default-Plate";
        console.error("Error: plate not valid");
      }
    }
  });
}

/**
 * Decorators on property -> with parameters -> configurable
 * @param v
 */
export function readonly(v: boolean) {
  return function(target: any, propertyKey: any) {
    let value: any;
    if (v) {
      Reflect.defineProperty(target, propertyKey, {
        configurable: true,
        enumerable: true,
        get: () => {
          // Return the scoped value
          return value;
        },
        set: (newValue: string) => {
          // Update the scoped value with max(newValue, min)
          if (!target["setted"]) {
            target["setted"] = true;
            value = newValue;
          }
        }
      });
    }
  };
}
/**
 * 
 * @param min length to be a valid value
 */
export function MinimumLength(min: number = 2) {
  // Scope the value to be reused
  let value: string;
  return (target: any, propertyKey: string | symbol) => {
    // Store the definition result
    const update = Reflect.defineProperty(target, propertyKey, {
      configurable: true,
      enumerable: true,
      get: () => {
        // Return the scoped value
        return value;
      },
      set: (newValue: string) => {
        // Update the scoped value with max(newValue, min)
        value = newValue.length >= min ? newValue : "no Valid";
      }
    });
    // If the update failed, something went wrong
    if (!update) {
      // Kill everything
      throw new Error("Unable to update property");
    }
  };
}