/**
 * Package: developodoFW, framework implemented in typescript to develop frontend web app
 * File: appGUI.ts
 * Description: it defines some utilities to craate web components
 * @autor: Carlos Serrano Sánchez for Developodo
 * @see pageController.ts
 * @see core.ts
 */
import {$} from './core';

/**
 * this class is instanciated in core class: App, so every app has all this methods available with sintax: this.gui.method() 
 */
export class AppGUI {
  /**
   * showLoading creates an on top div semitransparent with a spinner
   */
    showLoading() {
      this.hideLoading();
      let template = `<div class="develop-loading d-flex justify-content-center align-self-center" 
      style="width:100%;height:100%;position:absolute;top:0;left:0;z-index:9999;background-color: rgba(255,255,255,0.5);padding-top:45%">
            <div class="spinner-border text-primary" role="status">
              <span class="sr-only">Loading...</span>
            </div>
          </div>`;
      $("body").append(template);
    }
    /**
     * hideLoading hides the loading div
     * @see showLoading
     */
    hideLoading() {
      $(".develop-loading").remove();
    }
    /**
     * showToast creates a toast box on left bottom corner (if there are more than one, the new one is stacked bellow the older ones).
     * Toast are hidden automatically after 4 seconds.
     * @param msg the message to be shown on toast component
     */
    showToast(msg: string) {
      if (!$(".toast-container").length) {
        let container = `<div class="toast-container" aria-live="polite" aria-atomic="true" style="min-height: 200px;">
        <div style="position: absolute; bottom: 0; right: 0;">
          <!-- Then put toasts within -->
        </div>
      </div>    `;
        $("app").append(container);
      }
      let toastCode = Math.floor(Math.random() * 1000000) + 1;
      let template = `<div class="toast toast${toastCode}" role="alert" aria-live="assertive" aria-atomic="true">
                                  <div class="toast-header">
                                    <img src="resources/developodomini.png" class="rounded mr-2" alt="" style="width:20px">
                                    <strong class="mr-auto">Developodo</strong>
                                    <button type="button" class="ml-2 mb-1 close" data-dismiss="toast" aria-label="Close">
                                      <span aria-hidden="true">&times;</span>
                                    </button>
                                  </div>
                                  <div class="toast-body">
                                    ${msg}
                                  </div>
                                </div>`;
  
      $(".toast-container > div").append(template);
      $(".toast").toast({
        autohide: true,
        delay: 4000
      });
      setTimeout(() => {
        $(".toast" + toastCode).remove();
      }, 4500);  //0.5 seconds more to show de hiding animation and then toast is auto-destroyed
      $(".toast").toast("show");
    }
    /**
     * showModal creates a window modal on top of the page with the following parameters
     * @param title Title for the modal window
     * @param content HTML code to be inserted in the modal content div
     * @param positiveNameButton Message shown on the accept button
     * @param callbackPositive Function to be called when accept button is pressed
     */
    showModal(
      title: string,
      content: string,
      positiveNameButton: string,
      callbackPositive: any
    ) {
      let modalCode = Math.floor(Math.random() * 1000000) + 1;
      let template: string = `<div id="modal${modalCode}" class="modal fade" tabindex="-1" role="dialog" aria-hidden="true">
      <div class="modal-dialog" role="document">
        <div class="modal-content">
          <div class="modal-header">
            <h5 class="modal-title">${title}</h5>
            <button type="button" class="close" data-dismiss="modal" aria-label="Close">
              <span aria-hidden="true">&times;</span>
            </button>
          </div>
          <div class="modal-body">
          ${content}
          </div>
          <div class="modal-footer">
            <button type="button" class="btn btn-secondary" data-dismiss="modal">Close</button>
            <button type="button" class="btn btn-danger" id="modalButton${modalCode}">${positiveNameButton}</button>
          </div>
        </div>
      </div>
    </div>`;
      $("app").append(template);
      $("#modalButton" + modalCode).click(() => {
        callbackPositive();
        $("#modal" + modalCode).modal("hide");
      });
      $("#modal" + modalCode).modal("show");
    }
  }
