/**
 * Package: developodoFW, framework implemented in typescript to develop frontend web app
 * File: core.ts
 * Description: Import all dependecies: jquery, popper and bootstrap. This is the file that exported them and the defines 
 * the core class App and the decorator functions.
 * @autor: Carlos Serrano Sánchez for Developodo
 * @see pageController.ts
 * @see appGUI.ts
 */
import $ = require("jquery"); //npm install jquery --save
import "popper.js";  //npm install popper.js --save
import "bootstrap";  //npm install bootstrap --save
import  {AppGUI} from './appGUI';

/**
 * propertyBinding changes the value of the one HTML element's property when the variable assigned has new a new value.
 * @param field id of the HTML element
 * @param property name of the property to connect the variable assigned
 */
function propertyBinding(field: string,property:string) {
    return function(target: any, propertyKey: string) {
      let value = target[propertyKey]; //read de value of this variable
      Object.defineProperty(target, propertyKey, {
        //changing descriptor
        get: () => {
          return value;
        },
        set: newValue => {
          value = newValue;
          $(field).attr(property,value);
        }
      });
    };
  }
/**
 * twoWayBinding connect the value of an input field to a typescript variable bidirectionally
 * @param field id of the HTML element connected to
 */
function twoWayBinding(field: string) {
    App.bindOneWay(field);  //it allows onewaybinding too
    return function(target: any, propertyKey: string) {
      let value = target[propertyKey]; //read de value of this variable
      Object.defineProperty(target, propertyKey, {
        get: () => {
          return value;
        },
        set: newValue => {
          value = newValue;
          if ($(".tmpl-" + propertyKey)) $(".tmpl-" + propertyKey).html(value);
          $("#" + propertyKey).unbind("change"); //avoid infinite loop on next command
          $("#" + propertyKey).val(value);
          $("#" + field).change(x => {
            value = x.target.value;
            if ($(".tmpl-" + propertyKey)) $(".tmpl-" + propertyKey).html(value);
          });
        }
      });
    };
  }
/**
 * App core class -> onewaybinding and GUI tools
 */
  class App{
    gui:AppGUI;   //All controller class can access to gui in this way -> this.gui.showLoading(); , for example
    p: any;
    static executed: boolean = false;
    constructor(){
      this.gui=new AppGUI();
    }
    binding(p: any) {
      this.p = p;
      if (!App.executed) {  //if programmers define two (or more) Controller class (mistake) avoid executing binding method more than one time
        this.bindAllOneWay();
        App.executed = true;
      }
    }
    //This method is used by Decorators, programmers shouldn't use it
    static bindOneWay(field: string,container?:string) {
      container=(container?container:"app");
      let web = $(container).html();
      var replace = "{{(" + field + ")}}";
      var rex = new RegExp(replace, "g");
      web = web.replace(rex, '<span class="tmpl-$1"></span>');
      $(container).html(web);
    }
    /**
     * bindAllOneWay creates the connection between mustache sintax and variables (of controller class and initializated)
     * @param container optional, it defines the html container to analize, app by default
     */
    bindAllOneWay(container?:string) {
      container=(container?container:"app");
      let web = $(container).html();
      for (let item of Object.getOwnPropertyNames(this.p)) {
        if (
          typeof this.p[item] === "string" ||
          typeof this.p[item] === "number" ||
          typeof this.p[item] === "boolean"
        ) {
          var replace = "{{(" + item + ")}}";
          var rex = new RegExp(replace, "g");
          web = web.replace(rex, '<span class="tmpl-$1"></span>');
        }
      }
      $(container).html(web);
      for (let item of Object.getOwnPropertyNames(this.p)) {
        if (
          typeof this.p[item] === "string" ||
          typeof this.p[item] === "number" ||
          typeof this.p[item] === "boolean"
        ) {
          Object.defineProperty(this.p, item, {
            //changing descriptor
            set: newValue => {
              $(".tmpl-" + item).html(newValue);
            }
          });
        }
      }
    }
  }


  export { App, twoWayBinding,propertyBinding, $ };
