import React, { Component } from "react";
import "./App.css";
import "bootstrap/dist/css/bootstrap.min.css";
import EditItem from "./EditItem";
import { clientRestful } from "./client";
import { item } from "./items";
import * as enviroment from "./enviroment.json";
import MyModal from "./MyComponents/MyModal";

class App extends Component {
  myService: clientRestful;

  state = {
    selectedItem: {
      id: 0,
      title: "",
      description: ""
    },
    myitems: [],
    editItem: false,
    newItem: false
  };
  constructor(props: any) {
    super(props);

    this.onSubmit = this.onSubmit.bind(this); //child can access to this of this class
    this.onUpdateItem = this.onUpdateItem.bind(this);
    this.onCreateItem = this.onCreateItem.bind(this);
    this.onCancelEdit = this.onCancelEdit.bind(this);

    this.myService = new clientRestful(enviroment.endPoint);
  }

  componentDidMount() {
    this.getItems();
  }

  getItems() {
    this.myService.getItems().subscribe(items => {
      this.setState({ myitems: items });
    });
  }

  render() {
    const items = this.state.myitems;
    const editItem = this.state.editItem;

    if (!items) return null;
    const listItems = items.map((newitem: item) => (
      <tr key={newitem.id}>
        <td>{newitem.id}</td>
        <td>{newitem.title}</td>
        <td>{newitem.description}</td>
        <td>
          <button
            className="btn btn-info btn-edit"
            onClick={() => this.onEditItem(newitem)}
          >
            E
          </button>
          <button
            className="btn btn-danger btn-remove"
            onClick={() => this.onRemove(newitem.id)}
          >
            D
          </button>
        </td>
      </tr>
    ));

    return (
      <div className="App">
        <nav className="navbar navbar-light bg-light">
          <a className="navbar-brand" href="about">
            <img
              src="https://raw.githubusercontent.com/Developodo/home/master/developodomini.png"
              width="30"
              height="30"
              className="d-inline-block align-top"
              alt=""
            />
            Typescript(React) + Bootstrap (by Developodo)
          </a>
        </nav>
        <MyModal
          classButton="float-right"
          callback={this.onCreateItem}
          message={this.getForm()}
          title={"Add new Item"}
          openMessage={"Add"}
        />
        <button className="btn btn-success" onClick={() => this.onAddItem()}>
          Add
        </button>

        <table id="data" className="table table-striped">
          <thead>
            <tr>
              <th scope="col">#</th>
              <th scope="col">Title</th>
              <th scope="col">Descripion</th>
              <th scope="col">Actions</th>
            </tr>
          </thead>
          <tbody>{listItems}</tbody>
        </table>
        <hr />
        {editItem && (
          <EditItem
            onSubmit={this.onSubmit}
            onCancel={this.onCancelEdit}
            item={this.state.selectedItem}
            addItem={this.state.newItem}
          />
        )}
        <MyModal
          classButton="float-right margened "
          callback={this.onCreateItem}
          message={this.getForm()}
          title={"Add new Item (2)"}
          openMessage={"Another Add"}
        />
      </div>
    );
  }

  onEditItem(it: item) {
    this.setState({
      editItem: true,
      newItem: false,
      selectedItem: it
    });
  }
  onAddItem() {
    this.setState({
      editItem: true,
      newItem: true,
      selectedItem: {
        id: 0,
        title: "",
        description: ""
      }
    });
  }

  onRemove(id: string | number) {
    this.clearState();
    if (window.confirm("Remove it?")) {
      this.myService.removeItem(id).subscribe(() => {
        this.getItems();
      });
    }
  }

  onCancelEdit() {
    this.setState({
      editItem: false,
      newItem: false
    });
  }

  onSubmit(i: item) {
    if (this.state.newItem) {
      this.onCreateItem(i);
    } else {
      this.onUpdateItem(i);
    }
  }

  onUpdateItem(uitem: item) {
    console.log(uitem);
    this.myService
      .updateItem(uitem.id, uitem.title, uitem.description)
      .subscribe(i => {
        this.clearState();
        this.getItems();
      });
  }
  onCreateItem(uitem: item) {
    console.log(uitem);
    this.myService.createItem(uitem.title, uitem.description).subscribe(i => {
      this.clearState();
      this.getItems();
    });
    //this.clearState();
  }
  clearState() {
    this.setState({
      editItem: false,
      newItem: null
    });
  }

  getForm(): any {
    let html = (
      <div>
        <div className="form-group">
          <label>Title: </label>
          <input
            className="form-control"
            defaultValue=""
            name="title"
            required
            placeholder="item title"
          />
        </div>

        <div className="form-group form-row">
          <label>Description: </label>
          <input
            className="form-control"
            defaultValue=""
            name="description"
            required
            placeholder="item description"
          />
        </div>
      </div>
    );
    return html;
  }
}

export default App;
