import { Component, OnInit } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { AuthService } from "../../services/auth.service";
import { Router } from "@angular/router";

@Component({
  selector: "app-login",
  templateUrl: "./login.component.html",
  styleUrls: ["./login.component.css"]
})
export class LoginComponent implements OnInit {
  public registerForm: FormGroup;
  public submited: boolean;
  public submitting: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private auth: AuthService,
    private router: Router
  ) {
    this.submitting = false;
  }

  ngOnInit() {
    if (this.auth.isLogged()) {
      console.log("login se pira para home");
      this.router.navigate(["home"]);
    }

    this.registerForm = this.formBuilder.group({
      email: ["", [Validators.required, Validators.email]],
      password: ["", [Validators.required, Validators.minLength(6)]]
    });
    this.submited = false;
  }
  public isLogging() {
    return this.submitting;
  }
  // convenience getter for easy access to form fields
  public get f() {
    return this.registerForm.controls;
  }

  onSubmit() {
    if (this.submitting) {
      return;
    }

    this.submited = true;
    this.submitting = true;
    // stop here if form is invalid
    if (this.registerForm.invalid) {
      console.log("Invalid Form");
      this.submitting = false;
      return;
    }

    this.auth
      .login(this.f.email.value, this.f.password.value)
      .then(res => {
        console.log(res);
        this.router.navigate(["/home"]);
        this.submitting = false;
      })
      .catch(err => {
        alert(err);
        this.submitting = false;
      });
  }

  public logingGoogle() {
    this.submitting = true;
    this.auth
      .loginGoogle()
      .then(res => {
        this.submitting = false;
        this.router.navigate(["/home"]);
      })
      .catch(err => {
        this.submitting = false;
        alert(err);
        console.log(err);
      });
  }
}
