import Vue from 'vue';
import App from './App.vue';
import router from './router';
import store from './store';
import './registerServiceWorker';
// Vuetify
import Vuetify from 'vuetify';
import 'vuetify/dist/vuetify.min.css';
// Custom Stencil Component
import { applyPolyfills, defineCustomElements } from 'histencil/loader';

/** VUE NOTIFICATION */
import * as firebase from 'firebase';
import axios from 'axios';
import * as config from './config';

firebase.initializeApp(config);

try {
  const messaging = firebase.messaging();
  Vue.prototype.$messaging = firebase.messaging();
  // tslint:disable-next-line: max-line-length
  messaging.usePublicVapidKey(
    config.pairKey
  ); // 1. Generate a new key pair
  messaging 
    .requestPermission()
    .then(() => firebase.messaging().getToken())
    .then((token:any) => {
      axios
        .post(
          'https://iid.googleapis.com/iid/v1/' + token + '/rel/topics/all',
          {},
          {
            headers: {
              // key-server from console
              Authorization:
                'key='+config.serverKey
            },
          },
        )
        .then((r: any) => {
          console.log('Subscribed to all');
          new Vue({
            router,
            store,
            render: (h) => h(App),
            vuetify: new Vuetify(vuetifyOptions),
          }).$mount('#app');
        })
        .catch((err: any) => {
          console.log(err);
        });
    })
    .catch(function(err:any) {
      console.log('Unable to get permission to notify.', err);
    });
} catch (eer) {
  // Error with API and browser version
  console.log(eer);
}

Vue.config.productionTip = false;

const vuetifyOptions = {};
Vue.use(Vuetify);

// Binding stencil component to window
applyPolyfills().then(() => {
  defineCustomElements(window);
});


