import Vue from 'vue';
import Vuex from 'vuex';
import VuexPersist from 'vuex-persist';

import { item } from './model/item';

const vuexPersist = new VuexPersist({
  key: 'my-app',   // key used in localstorage
  storage: localStorage   //Change localStorage with sessionStorage to use that other storage system 
})

Vue.use(Vuex);


export default new Vuex.Store({
  state: {
    todos: Array<item>()
  },
  mutations: {
    addItem(context:any,n:item){
      context.todos.push(n);
    },
    editItem(context:any, n) {
      const index = context.todos.indexOf(n.selected, 0);
      if ( index >= 0 ) {
        context.todos[index].message = n.item.message;
        context.todos[index].image = n.item.image;
      }
    },
    removeItem(context:any, n:item ) {
      const index = context.todos.indexOf(n, 0);
      if (index > -1) {
        context.todos.splice(index, 1);
      }
    }
  },
  actions: {
    addItemAsync(context:any, n:item){
      return new Promise((resolve,reject)=>{
        setTimeout(()=>{
          context.commit('addItem', n);
          resolve();
        },2000)
      })
    }
  },
  plugins: [vuexPersist.plugin]  // <- IMPORTANT
});
