import { Component, OnInit, ViewChild } from '@angular/core';
import { AuthService } from '../../services/auth.service';
import { Entry } from '../../model/entry';
import { EntriesService } from '../../services/entries.service';
import {
  IonInfiniteScroll,
  ModalController,
  AlertController
} from '@ionic/angular';
import { EditPageComponent } from '../../customComponents/edit-page/edit-page.component';
import { IonVirtualScroll } from '@ionic/angular';
import { ViewImageComponent } from 'src/app/customComponents/view-image/view-image.component';
import { CommonsService } from '../../services/commons.service';

@Component({
  selector: 'app-list-images',
  templateUrl: './list-images.component.html',
  styleUrls: ['./list-images.component.scss']
})
export class ListImagesComponent implements OnInit {
  @ViewChild('infiniteScroll') ionInfiniteScroll: IonInfiniteScroll;
  @ViewChild(IonVirtualScroll) virtualScroll: IonVirtualScroll;
  resetInfinityScroll: IonInfiniteScroll;
  myEntries: Entry[] = [];

  constructor(
    public auth: AuthService,
    public cloud: EntriesService,
    public modalController: ModalController,
    private alertController: AlertController,
    private commonS: CommonsService
  ) { }

  ngOnInit() {
    this.loadEntries(); 
    // important to do it here, when dom is made. infinite scroll need this solution
  }
  public logOut() {
    this.auth.logOut();
  }

  public async loadEntries() {
    this.commonS.showLoading();
    this.myEntries = await this.cloud.getEntries(true);
    // next line is very important to update dinamic content into virtualscroll
    this.virtualScroll.checkEnd();
    this.commonS.hideLoading();
    // hide loading
  }
  public async refreshEntries(e) {
    await this.loadEntries();
    // close the refresher
    e.target.complete();
  }
  public async loadMoreEntries(e) {
    this.commonS.showLoading();
    const newEntries = await this.cloud.getEntries(false); // to add more
    if (newEntries.length > 0) {
      this.myEntries = this.myEntries.concat(newEntries);
      this.virtualScroll.checkEnd();
    }
    this.commonS.hideLoading();
    e.target.complete();
  }

  async editModal(e: Entry) {
    const modal = await this.modalController.create({
      component: EditPageComponent,
      componentProps: e
    });
    modal.onDidDismiss().then(d => {
      if (d && d.data && d.data.r) {
        this.loadEntries();
      }
    });
    return await modal.present();
  }

  async viewImage(e: Entry) {
    const modal = await this.modalController.create({
      component: ViewImageComponent,
      componentProps: { entry: e }
    });
    modal.onDidDismiss().then(d => { });
    return await modal.present();
  }

  public removeEntry(e) {
    this.presentAlertConfirm(e);
  }

  async presentAlertConfirm(e) {
    const alert = await this.alertController.create({
      header: 'Confirm!',
      message: 'Remove <strong>' + e.title + '</strong>!!!',
      buttons: [
        {
          text: 'Cancel',
          role: 'cancel',
          cssClass: 'secondary',
          handler: blah => {
            console.log('Confirm Cancel: blah');
          }
        },
        {
          text: 'Okay',
          handler: () => {
            this.commonS.showLoading();
            this.cloud.removeEntry(e).then(() => {
              this.commonS.hideLoading();
              this.loadEntries();
            });
          }
        }
      ]
    });

    await alert.present();
  }
}
